[uwsgi]
autoload=false
master=true
workers=5
max-requests=5000
max-worker-lifetime=3600
reload-on-rss=200
no-orphans=true
{% if is_development %}
http=0.0.0.0:8001
{% else %}
socket=127.0.0.1:8002
{% endif %}
log-date=true
uid=root
gid=root
venv={{ venv_dir }}
module=packy_agent.control_server.wsgi
callable=application
pidfile={{ uwsgi_pid_file }}
vacuum=true
env=
close-on-exec=true

# enable-threads=true - to let Sentry work correctly
enable-threads=true
