DEFAULT_SERVER_BASE_URL = 'https://p04.packy.io/'
PACKY_SERVER_BASE_URL_ENVVAR_NAME = 'PACKY_SERVER_BASE_URL'
AGENT_CONFIG_ENVVAR_NAME = 'PACKY_AGENT_CONFIG'
DEFAULT_AGENT_CONFIG_PATH = '/etc/packy-agent.yaml'
CONTROL_SERVER_CONFIG_ENVVAR_NAME = 'PACKY_CONTROL_SERVER_CONFIG'
DEFAULT_CONTROL_SERVER_CONFIG_PATH = '/etc/packy-control-server.yaml'
