{% extends "layout.html" %}
{% block content %}
          <div class="row">
            <div class="col-xs-12 col-md-6 col-lg-6">
              <div class="panel panel-default panel-border-color panel-border-color-primary">
                <div class="panel-heading panel-heading-divider">Reset Configuration</div>
                <div class="panel-body">
                        <form method="post" novalidate="" class="form-horizontal group-border-dashed">
                          {{ form.csrf_token }}
                            <p class="text-center">
                              <button type="submit" name="action" value="deactivate" class="btn btn-space btn-primary">Deactivate</button>
                              {% if not is_inside_docker_container %}
                                <button type="submit" name="action" value="reset_network" class="btn btn-space btn-primary">Reset Network Configuration and reboot</button>
                              {% endif %}
                              <button type="submit" name="action" value="deactivate_and_reset" class="btn btn-space btn-primary">Deactivate, reset all and reboot</button>
                            </p>
                        </form>
                </div>
              </div>
            </div>
          </div>
{% endblock %}
