{% extends "layout.html" %}
{% block content %}
          <div class="row">
            <div class="col-xs-12 col-md-6 col-lg-3">
                        <div class="widget widget-tile">
                          <div class="data-info">
                            <div class="desc">Agent status</div>
                            <div class="value">{{ agent_status }}</div>
                          </div>
                        </div>
            </div>
          </div>
{% endblock %}