[Unit]
Description=Packy
Documentation=
After=network.target

[Service]
ExecStartPre=/bin/mkdir -p {{ supervisor_run_directory }}
ExecStartPre=/bin/chmod a+rwx {{ supervisor_run_directory }}
ExecStart={{ supervisord_executable }} -n -c {{ supervisor_configuration_file }}
ExecStop={{ supervisorctl_executable }} -c {{ supervisor_configuration_file }} $OPTIONS shutdown
ExecReload={{ supervisorctl_executable }} -c {{ supervisor_configuration_file }} $OPTIONS reload
KillMode=process
Restart=on-failure
RestartSec=50s

[Install]
WantedBy=multi-user.target
