{% extends "layout.html" %}
{% from "_formhelpers.html" import render_field %}
{% block content %}
        {% for log_title, log_filename, log_content in logs %}
          <div class="row">
            <div class="col-xs-12">
              <div class="panel panel-default panel-border-color panel-border-color-primary">
                <div class="panel-heading panel-heading-divider">{{ log_title }}<span class="panel-subtitle">{{ log_filename }}</span></div>
                <div class="panel-body">
<pre><code>
{{ log_content }}
</code></pre>
                </div>
              </div>
            </div>
          </div>
        {% endfor %}
          <div class="row">
            <div class="col-xs-12 col-md-6 col-lg-6">
              <div class="panel panel-default panel-border-color panel-border-color-primary">
                <div class="panel-heading panel-heading-divider">Packy Agent local configuration file<span class="panel-subtitle">{{ packy_agent_configuration_file }}</span></div>
                <div class="panel-body">
<pre><code>
{{ packy_agent_configuration_file_content }}
</code></pre>
                </div>
              </div>
            </div>
            <div class="col-xs-12 col-md-6 col-lg-6">
              <div class="panel panel-default panel-border-color panel-border-color-primary">
                <div class="panel-heading panel-heading-divider">Packy Agent Custom configuration</div>
                <div class="panel-body">
<pre><code>
{{ packy_agent_custom_configuration }}
</code></pre>
                </div>
              </div>
            </div>
          </div>
          <div class="row">
            <div class="col-xs-12 col-md-6 col-lg-6">
              <div class="panel panel-default panel-border-color panel-border-color-primary">
                <div class="panel-heading panel-heading-divider">Packy Agent Celery configuration</div>
                <div class="panel-body">
<pre><code>
{{ packy_agent_celery_configuration }}
</code></pre>
                </div>
              </div>
            </div>
          </div>
          <div class="row">
            <div class="col-xs-12 col-md-6 col-lg-6">
              <div class="panel panel-default panel-border-color panel-border-color-primary">
                <div class="panel-heading panel-heading-divider">Control Server local configuration file<span class="panel-subtitle">{{ control_server_configuration_file }}</span></div>
                <div class="panel-body">
<pre><code>
{{ control_server_configuration_file_content }}
</code></pre>
                </div>
              </div>
            </div>
            <div class="col-xs-12 col-md-6 col-lg-6">
              <div class="panel panel-default panel-border-color panel-border-color-primary">
                <div class="panel-heading panel-heading-divider">Control Server Custom configuration</div>
                <div class="panel-body">
<pre><code>
{{ control_server_custom_configuration }}
</code></pre>
                </div>
              </div>
            </div>
          </div>
          <div class="row">
            <div class="col-xs-12 col-md-6 col-lg-6">
              <div class="panel panel-default panel-border-color panel-border-color-primary">
                <div class="panel-heading panel-heading-divider">Control Server Flask configuration</div>
                <div class="panel-body">
<pre><code>
{{ control_server_flask_configuration }}
</code></pre>
                </div>
              </div>
            </div>
          </div>
{% endblock %}
