{% extends "layout.html" %}
{% block content %}
          <div class="row">
            <div class="col-xs-12 col-md-6 col-lg-6">
              <div class="panel panel-default panel-border-color panel-border-color-primary">
                <div class="panel-heading panel-heading-divider">Success</div>
                <div class="panel-body">
                  {% for message in get_flashed_messages() %}
                    <div role="alert" class="alert alert-primary alert-dismissible">
                      <button type="button" data-dismiss="alert" aria-label="Close" class="close">
                        <span aria-hidden="true" class="mdi mdi-close"></span></button>
                      <span class="icon mdi mdi-info-outline"></span>{{ message }}
                    </div>
                  {% endfor %}
                  <div>
                    <p class="text-center">
                      {{ message }}
                    </p>
                  </div>
                  <div>
                    <p class="text-center">
                      <a href="{{ back_url }}" class="btn btn-space btn-primary">{{ button_text }}</a>
                    </p>
                  </div>
                </div>
              </div>
            </div>
          </div>
{% endblock %}
