#!/usr/bin/env bash

rm -f /tmp/packy-upgrade-in-progress.lock
# TODO(dmu) LOW: Deduplicate with `packy` service definition (maybe run recover or this entire shell script under supervisor)
SHELL=/bin/bash {{ recover_script_path }} >> /var/log/packy-recover.log 2>&1
{{ venv_bin_dir }}/supervisord -n -c {{ control_server_configuration.supervisor_configuration_file }}
